package com.fossgalaxy.games.tbs.ai.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.ai.Pathfinder;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.ui.GameAction;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.util.List;
import java.util.ListIterator;

public abstract class FirstLegalRule extends PerEntityRule {

    protected String category;

    public FirstLegalRule(String category) {
        this.category = category;
    }


    @Override
    public boolean isForEntity(GameState state, Entity entity) {
        return true;
    }

    @Override
    public Order generateOrder(GameState state, Entity entity) {

        CubeCoordinate coordinate = chooseTarget(entity, state);
        if (coordinate == null) {
            return null;
        }

        List<GameAction> actions = entity.getType().getAvailableActions();
        for (GameAction action : actions) {

            if (!category.equals(action.getCategory())) {
                continue;
            }


            if (action.isPossible(entity, state, coordinate)) {
                return action.generateOrder(coordinate, state);
            }

        }

        return null;
    }

    public GameAction getActionFor(String category, GameState state, Entity entity) {

        List<GameAction> actions = entity.getType().getAvailableActions();
        for (GameAction action : actions) {

            if (category.equals(action.getCategory()) && action.isPossible(entity, state)) {
                return action;
            }

        }

        return null;
    }

    protected Order moveTo(GameState state, CubeCoordinate target, Entity entity){

        GameAction action = getActionFor("move", state, entity);
        if (action == null) {
            return null;
        }

        List<CubeCoordinate> co = Pathfinder.findPath(state, entity, target);
        if (co == null) {
            return null;
        }

        ListIterator<CubeCoordinate> pathIterator = co.listIterator(co.size());
        while(pathIterator.hasPrevious()) {
            CubeCoordinate prev = pathIterator.previous();

            if (action.isPossible(entity, state, prev)) {
                return action.generateOrder(prev, state);
            }


        }

        return null;
    }

    protected CubeCoordinate chooseTarget(Entity entity, GameState state) {
        return entity.getPos();
    }
}
