package com.fossgalaxy.games.tbs.ai.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.actions.AbstractAction;
import com.fossgalaxy.games.tbs.ai.Pathfinder;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.entity.HexagonTile;
import com.fossgalaxy.games.tbs.entity.Resource;
import com.fossgalaxy.games.tbs.order.MoveOrder;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.parameters.ResourceType;
import com.fossgalaxy.object.annotations.ObjectDef;
import org.codetome.hexameter.core.api.CubeCoordinate;
import org.codetome.hexameter.core.api.Hexagon;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by pwillic on 18/05/2018.
 */
public class RunTowardsResource extends MovementRule {

    private final List<ResourceType> targets;

    @ObjectDef("RunTowardsResource")
    public RunTowardsResource(ResourceType... targets) {
        this.targets = Arrays.asList(targets);
    }

    @Override
    protected CubeCoordinate chooseTarget(Entity entity, GameState gameState) {
        CubeCoordinate ourPos = entity.getPos();
        Collection<Resource> resources = gameState.getResources().stream()
                .filter(x -> targets.contains(x.getType()))
                .collect(Collectors.toList());

        if (resources.isEmpty()) return null; // Unlikely but still
        Resource closestResource = null;
        CubeCoordinate nextMove = null;
        double closestDist = Double.MAX_VALUE;
        for (Resource resource : resources) {
            Entity onResource = gameState.getEntityAt(resource.getLocation());
            if (onResource != null) continue;
            List<CubeCoordinate> path = Pathfinder.findPath(gameState, entity, resource.getLocation());
            if (path == null) continue;
            if (path.size() <= 2) continue;
            CubeCoordinate pos = path.get(path.size() - 2);
            double distance = gameState.getDistance(ourPos, pos);
            if (distance == 0) continue;
            if (distance < closestDist) {
                closestDist = distance;
                closestResource = resource;
                nextMove = path.get(1);
            }
        }
        if (closestResource == null) return null;

        return nextMove;
    }
}
