package com.fossgalaxy.games.tbs.ai.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.parameters.EntityType;
import com.fossgalaxy.games.tbs.ui.GameAction;
import com.fossgalaxy.object.annotations.ObjectDef;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by pwillic on 18/05/2018.
 */
public class UseActionOnEntity extends PerEntityRule {
    private final GameAction action;
    private final List<EntityType> targets;

    @ObjectDef("UseActionOnEntity")
    public UseActionOnEntity(GameAction action, List<EntityType> targets) {
        this.action = action;
        this.targets = targets;
    }

    @Override
    public boolean isForEntity(GameState state, Entity entity) {
        return entity.getType().getAvailableActions().contains(action);
    }

    @Override
    public Order generateOrder(GameState gameState, Entity entity) {
        CubeCoordinate ourPos = entity.getPos();
        Collection<Entity> entities = gameState.getEntities().stream()
                .filter(x -> x.getOwner() != entity.getOwner())
                .filter(x -> targets.contains(x.getType()))
                .collect(Collectors.toList());

        if(entities.isEmpty()) return null;

        Entity closestEntity = null;
        double closestDist = Double.MAX_VALUE;
        for(Entity other : entities){
            CubeCoordinate theirPos = other.getPos();
            if(!action.isPossible(entity, gameState, theirPos)) continue;
            double distance = gameState.getDistance(ourPos, theirPos);
            if(distance == 0) continue;
            if(distance < closestDist){
                closestDist = distance;
                closestEntity = other;
            }
        }

        if(closestEntity == null) return null;
        return action.generateOrder(closestEntity.getPos(), gameState);
    }
}
